/*
 * Copyright (C) 2011-2016 Intel Corporation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of Intel Corporation nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef _AE_REPORT_ATTESTATION_REQUEST_H
#define _AE_REPORT_ATTESTATION_REQUEST_H

#include <IAERequest.h>

class AEReportAttestationRequest : public IAERequest
{
    public:
        AEReportAttestationRequest();
        AEReportAttestationRequest(uint32_t platformInfoLength, const uint8_t* platformInfo, uint32_t attestation_error_code, uint32_t updateInfoLength, uint32_t timeout = 0);
        AEReportAttestationRequest(const AEReportAttestationRequest& other);
        ~AEReportAttestationRequest();

        AEMessage* serialize(ISerializer* serializer);
        void inflateValues(uint32_t platformInfoLength, const uint8_t* platformInfo, uint32_t attestation_error_code, uint32_t updateInfoLength, uint32_t timeout = 0);

        //getters
        inline uint32_t GetAttestationErrorCode() const { return mAttestationErrorCode; }
        inline uint32_t GetPlatformInfoLength() const { return mPlatformInfoLength;}
        inline const uint8_t* GetPlatformInfo() const { return mPlatformInfo;}
        inline uint32_t GetUpdateInfoLength() const { return mUpdateInfoLength;}

        //operators
        bool operator==(const AEReportAttestationRequest& other) const;
        AEReportAttestationRequest& operator=(const AEReportAttestationRequest& request);

        //checks
        bool check();

        //hooks
        IAEResponse* execute(IAESMLogic* aesmLogic);

        //used to determin in which queue to be placed
        virtual RequestClass getRequestClass();
        void visit(IAERequestVisitor& visitor);

    protected:
        void ReleaseMemory();
        void CopyFields(uint32_t platformInfoLength, const uint8_t* platformInfo, uint32_t attestation_error_code, uint32_t updateInfoLength, uint32_t timeout);

        uint32_t    mAttestationErrorCode;

        uint32_t    mPlatformInfoLength;
        uint8_t*    mPlatformInfo;
        uint32_t    mUpdateInfoLength;
};

#endif
